@@ -34,11 +34,16 @@ private Q_SLOTS:
     void test();
 
 public:
-    KUrlCompletionTest() {}
+    KUrlCompletionTest() {
+#ifdef NO_WAIT // kurlcompletiontest-nowait sets this, to test what happens on slower systems (or systems with many dirs or users)
+        qputenv("KURLCOMPLETION_WAIT", "1"); // 1ms, too short for a full listing of /usr/bin, but at least give a chance for a few items in the result
+#endif
+    }
     ~KUrlCompletionTest()
     {
         teardown();
     }
+    void runAllTests();
     void setup();
     void teardown();
     void testLocalRelativePath();
@@ -47,9 +52,12 @@ public:
     void testEmptyCwd();
     void testBug346920();
     void testUser();
+    void testCancel();
+
+    // remember to register new test methods in runAllTests
 
 private:
-    void waitForCompletion();
+    void waitForCompletion(KUrlCompletion *completion);
     KUrlCompletion *m_completion;
     QTemporaryDir *m_tempDir;
     QUrl m_dirURL;
@@ -95,18 +103,21 @@ void KUrlCompletionTest::setup()
 void KUrlCompletionTest::teardown()
 {
     delete m_completion;
-    m_completion = 0;
+    m_completion = nullptr;
     delete m_tempDir;
-    m_tempDir = 0;
+    m_tempDir = nullptr;
     delete m_completionEmptyCwd;
-    m_completionEmptyCwd = 0;
+    m_completionEmptyCwd = nullptr;
 }
-void KUrlCompletionTest::waitForCompletion()
+
+void KUrlCompletionTest::waitForCompletion(KUrlCompletion *completion)
 {
-    while (m_completion->isRunning()) {
+    while (completion->isRunning()) {
         qDebug() << "waiting for thread...";
-        QThread::usleep(10);
+        QTest::qWait(5);
     }
+    // The thread emitted a signal, process it.
+    qApp->sendPostedEvents(nullptr, QEvent::MetaCall);
 }
 
 void KUrlCompletionTest::testLocalRelativePath()
@@ -114,7 +125,7 @@ void KUrlCompletionTest::testLocalRelativePath()
     qDebug();
     // Completion from relative path, with two matches
     m_completion->makeCompletion(QStringLiteral("f"));
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QStringList comp1all = m_completion->allMatches();
     qDebug() << comp1all;
     QCOMPARE(comp1all.count(), 4);
@@ -128,7 +139,7 @@ void KUrlCompletionTest::testLocalRelativePath()
     // Completion from relative path
     qDebug() << endl << "now completing on 'file#'";
     m_completion->makeCompletion(QStringLiteral("file#"));
-    waitForCompletion();
+    QVERIFY(!m_completion->isRunning()); // last listing reused
     QStringList compall = m_completion->allMatches();
     qDebug() << compall;
     QCOMPARE(compall.count(), 1);
@@ -139,7 +150,7 @@ void KUrlCompletionTest::testLocalRelativePath()
     // Completion with empty string
     qDebug() << endl << "now completing on ''";
     m_completion->makeCompletion(QLatin1String(""));
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QStringList compEmpty = m_completion->allMatches();
     QCOMPARE(compEmpty.count(), 0);
 
@@ -148,7 +159,7 @@ void KUrlCompletionTest::testLocalRelativePath()
     // fixed in https://codereview.qt-project.org/143134.
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 1)
     m_completion->makeCompletion(".");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compAllHidden = m_completion->allMatches();
     QCOMPARE(compAllHidden.count(), 2);
     QVERIFY(compAllHidden.contains(".1_hidden_file_subdir/"));
@@ -157,14 +168,14 @@ void KUrlCompletionTest::testLocalRelativePath()
 
     // Completion with '.2', should find only hidden folders starting with '2'
     m_completion->makeCompletion(".2");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compHiddenStartingWith2 = m_completion->allMatches();
     QCOMPARE(compHiddenStartingWith2.count(), 1);
     QVERIFY(compHiddenStartingWith2.contains(".2_hidden_file_subdir/"));
 
     // Completion with 'file.', should only find one file
     m_completion->makeCompletion("file.");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compFileEndingWithDot = m_completion->allMatches();
     QCOMPARE(compFileEndingWithDot.count(), 1);
     QVERIFY(compFileEndingWithDot.contains("file."));
@@ -175,7 +186,7 @@ void KUrlCompletionTest::testLocalAbsolutePath()
     // Completion from absolute path
     qDebug() << m_dir + "file#";
     m_completion->makeCompletion(m_dir + "file#");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QStringList compall = m_completion->allMatches();
     qDebug() << compall;
     QCOMPARE(compall.count(), 1);
@@ -187,7 +198,7 @@ void KUrlCompletionTest::testLocalAbsolutePath()
     // Completion with '.', should find all hidden folders
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 1)
     m_completion->makeCompletion(m_dir + ".");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compAllHidden = m_completion->allMatches();
     QCOMPARE(compAllHidden.count(), 2);
     QVERIFY(compAllHidden.contains(m_dir + ".1_hidden_file_subdir/"));
@@ -196,14 +207,14 @@ void KUrlCompletionTest::testLocalAbsolutePath()
 
     // Completion with '.2', should find only hidden folders starting with '2'
     m_completion->makeCompletion(m_dir + ".2");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compHiddenStartingWith2 = m_completion->allMatches();
     QCOMPARE(compHiddenStartingWith2.count(), 1);
     QVERIFY(compHiddenStartingWith2.contains(m_dir + ".2_hidden_file_subdir/"));
 
     // Completion with 'file.', should only find one file
     m_completion->makeCompletion(m_dir + "file.");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compFileEndingWithDot = m_completion->allMatches();
     QCOMPARE(compFileEndingWithDot.count(), 1);
     QVERIFY(compFileEndingWithDot.contains(m_dir + "file."));
@@ -215,7 +226,7 @@ void KUrlCompletionTest::testLocalURL()
     qDebug();
     QUrl url = QUrl::fromLocalFile(m_dirURL.toLocalFile() + "file");
     m_completion->makeCompletion(url.toString());
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QStringList comp1all = m_completion->allMatches();
     qDebug() << comp1all;
     QCOMPARE(comp1all.count(), 4);
@@ -236,7 +247,7 @@ void KUrlCompletionTest::testLocalURL()
     qDebug() << "makeCompletion(" << url << ")";
     QString comp2 = m_completion->makeCompletion(url.toString());
     QVERIFY(comp2.isEmpty());
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QVERIFY(m_completion->allMatches().isEmpty());
 
     // Completion from URL with a ref -> no match
@@ -244,14 +255,14 @@ void KUrlCompletionTest::testLocalURL()
     url.setFragment(QStringLiteral("ref"));
     qDebug() << "makeCompletion(" << url << ")";
     m_completion->makeCompletion(url.toString());
-    waitForCompletion();
+    waitForCompletion(m_completion);
     QVERIFY(m_completion->allMatches().isEmpty());
 
     // Completion with '.', should find all hidden folders
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 1)
     qDebug() << "makeCompletion(" << (m_dirURL.toString() + ".") << ")";
     m_completion->makeCompletion(m_dirURL.toString() + ".");
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compAllHidden = m_completion->allMatches();
     QCOMPARE(compAllHidden.count(), 2);
     QVERIFY(compAllHidden.contains(m_dirURL.toString() + ".1_hidden_file_subdir/"));
@@ -262,7 +273,7 @@ void KUrlCompletionTest::testLocalURL()
     url = QUrl::fromLocalFile(m_dirURL.toLocalFile() + ".2");
     qDebug() << "makeCompletion(" << url << ")";
     m_completion->makeCompletion(url.toString());
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compHiddenStartingWith2 = m_completion->allMatches();
     QCOMPARE(compHiddenStartingWith2.count(), 1);
     QVERIFY(compHiddenStartingWith2.contains(m_dirURL.toString() + ".2_hidden_file_subdir/"));
@@ -271,7 +282,7 @@ void KUrlCompletionTest::testLocalURL()
     url = QUrl::fromLocalFile(m_dirURL.toLocalFile() + "file.");
     qDebug() << "makeCompletion(" << url << ")";
     m_completion->makeCompletion(url.toString());
-    waitForCompletion();
+    waitForCompletion(m_completion);
     const auto compFileEndingWithDot = m_completion->allMatches();
     QCOMPARE(compFileEndingWithDot.count(), 1);
     QVERIFY(compFileEndingWithDot.contains(m_dirURL.toString() + "file."));
@@ -283,7 +294,7 @@ void KUrlCompletionTest::testEmptyCwd()
     // Completion with empty string (with a KUrlCompletion whose cwd is "")
     qDebug() << endl << "now completing on '' with empty cwd";
     m_completionEmptyCwd->makeCompletion(QLatin1String(""));
-    waitForCompletion();
+    waitForCompletion(m_completionEmptyCwd);
     QStringList compEmpty = m_completionEmptyCwd->allMatches();
     QCOMPARE(compEmpty.count(), 0);
 }
@@ -291,7 +302,7 @@ void KUrlCompletionTest::testEmptyCwd()
 void KUrlCompletionTest::testBug346920()
 {
     m_completionEmptyCwd->makeCompletion(QStringLiteral("~/."));
-    waitForCompletion();
+    waitForCompletion(m_completionEmptyCwd);
     m_completionEmptyCwd->allMatches();
     // just don't crash
 }
@@ -299,14 +310,67 @@ void KUrlCompletionTest::testBug346920()
 void KUrlCompletionTest::testUser()
 {
     m_completionEmptyCwd->makeCompletion(QStringLiteral("~"));
-    waitForCompletion();
+    waitForCompletion(m_completionEmptyCwd);
     const auto matches = m_completionEmptyCwd->allMatches();
-    foreach(const auto& user, KUser::allUserNames()) {
-        QVERIFY(matches.contains(QLatin1Char('~') + user));
+    if (!KUser::allUserNames().isEmpty()) {
+        Q_ASSERT(!matches.isEmpty());
+    }
+    foreach (const auto &user, KUser::allUserNames()) {
+        QVERIFY2(matches.contains(QLatin1Char('~') + user), qPrintable(matches.join(' ')));
+    }
+
+    // Check that the same query doesn't re-list
+    m_completionEmptyCwd->makeCompletion(QStringLiteral("~"));
+    QVERIFY(!m_completionEmptyCwd->isRunning());
+    QCOMPARE(m_completionEmptyCwd->allMatches(), matches);
+}
+
+// Test cancelling a running thread
+// In a normal run (./kurlcompletiontest) and a reasonable amount of files, we have few chances of making this happen
+// But in a "nowait" run (./kurlcompletiontest-nowait), this will cancel the thread before it even starts listing the dir.
+void KUrlCompletionTest::testCancel()
+{
+    KUrlCompletion comp;
+    comp.setDir(QUrl::fromLocalFile("/usr/bin"));
+    comp.makeCompletion(QStringLiteral("g"));
+    const QStringList matchesG = comp.allMatches();
+    // We get many matches in a normal run, and usually 0 matches when testing "no wait" (thread is sleeping) -> this is where this method can test cancelling
+    //qDebug() << "got" << matchesG.count() << "matches";
+    bool done = !comp.isRunning();
+
+    // Doing the same search again, should hopefully not restart everything from scratch
+    comp.makeCompletion(QStringLiteral("g"));
+    const QStringList matchesG2 = comp.allMatches();
+    QVERIFY(matchesG2.count() >= matchesG.count());
+    if (done) {
+        QVERIFY(!comp.isRunning()); // it had no reason to restart
+    }
+    done = !comp.isRunning();
+
+    // Search for something else, should reuse dir listing but not mix up results
+    comp.makeCompletion(QStringLiteral("a"));
+    if (done) {
+        QVERIFY(!comp.isRunning()); // it had no reason to restart
+    }
+    const QStringList matchesA = comp.allMatches();
+    //qDebug() << "got" << matchesA.count() << "matches";
+    foreach (const QString &match, matchesA) {
+        QVERIFY2(!match.startsWith('g'), qPrintable(match));
+    }
+    waitForCompletion(&comp);
+    foreach (const QString &match, comp.allMatches()) {
+        QVERIFY2(!match.startsWith('g'), qPrintable(match));
     }
 }
 
 void KUrlCompletionTest::test()
+{
+    runAllTests();
+    // Try again, with another QTemporaryDir (to check that the caching doesn't give us wrong results)
+    runAllTests();
+}
+
+void KUrlCompletionTest::runAllTests()
 {
     setup();
     testLocalRelativePath();
@@ -315,16 +379,7 @@ void KUrlCompletionTest::test()
     testEmptyCwd();
     testBug346920();
     testUser();
-    teardown();
-
-    // Try again, with another QTemporaryDir (to check that the caching doesn't give us wrong results)
-    setup();
-    testLocalRelativePath();
-    testLocalAbsolutePath();
-    testLocalURL();
-    testEmptyCwd();
-    testBug346920();
-    testUser();
+    testCancel();
     teardown();
 }
 
