@@ -20,6 +20,7 @@
 */
 
 #include "copyjob.h"
+#include "kiocoredebug.h"
 #include <errno.h>
 #include "kcoredirlister.h"
 #include "kfileitem.h"
@@ -80,6 +81,7 @@ enum DestinationState {
 
 /**
  * States:
+ *     STATE_INITIAL the constructor was called
  *     STATE_STATING for the dest
  *     statCurrentSrc then does, for each src url:
  *      STATE_RENAMING if direct rename looks possible
@@ -95,6 +97,7 @@ enum DestinationState {
  *     done.
  */
 enum CopyJobState {
+    STATE_INITIAL,
     STATE_STATING,
     STATE_RENAMING,
     STATE_LISTING,
@@ -131,7 +134,7 @@ public:
         , m_mode(mode)
         , m_asMethod(asMethod)
         , destinationState(DEST_NOT_STATED)
-        , state(STATE_STATING)
+        , state(STATE_INITIAL)
         , m_freeSpace(-1)
         , m_totalSize(0)
         , m_processedSize(0)
@@ -151,7 +154,7 @@ public:
         , m_bOverwriteAllFiles(false)
         , m_bOverwriteAllDirs(false)
         , m_conflictError(0)
-        , m_reportTimer(0)
+        , m_reportTimer(nullptr)
     {
     }
 
@@ -304,6 +307,9 @@ QUrl CopyJob::destUrl() const
 void CopyJobPrivate::slotStart()
 {
     Q_Q(CopyJob);
+    if (q->isSuspended()) {
+        return;
+    }
     if (m_mode == CopyJob::CopyMode::Move) {
         Q_FOREACH (const QUrl &url, m_srcList) {
             if (m_dest.scheme() == url.scheme() && m_dest.host() == url.host()) {
@@ -329,6 +335,7 @@ void CopyJobPrivate::slotStart()
     m_reportTimer->start(REPORT_TIMEOUT);
 
     // Stat the dest
+    state = STATE_STATING;
     const QUrl dest = m_asMethod ? m_dest.adjusted(QUrl::RemoveFilename) : m_dest;
     KIO::Job *job = KIO::stat(dest, StatJob::DestinationSide, 2, KIO::HideProgressInfo);
     qCDebug(KIO_COPYJOB_DEBUG) << "CopyJob: stating the dest" << m_dest;
@@ -531,6 +538,20 @@ bool CopyJob::doSuspend()
     return Job::doSuspend();
 }
 
+bool CopyJob::doResume()
+{
+    Q_D(CopyJob);
+    switch (d->state) {
+        case STATE_INITIAL:
+            QTimer::singleShot(0, this, SLOT(slotStart()));
+            break;
+        default:
+            // not implemented
+            break;
+    }
+    return Job::doResume();
+}
+
 void CopyJobPrivate::slotReport()
 {
     Q_Q(CopyJob);
@@ -603,7 +624,7 @@ void CopyJobPrivate::slotEntries(KIO::Job *job, const UDSEntryList &list)
 void CopyJobPrivate::slotSubError(ListJob *job, ListJob *subJob)
 {
     const QUrl url = subJob->url();
-    qWarning() << url << subJob->errorString();
+    qCWarning(KIO_CORE) << url << subJob->errorString();
 
     Q_Q(CopyJob);
 
@@ -668,7 +689,7 @@ void CopyJobPrivate::addCopyInfoFromUDSEntry(const UDSEntry &entry, const QUrl &
                 for (int n = 0; n < numberOfSlashes + 1; ++n) {
                     pos = path.lastIndexOf('/', pos - 1);
                     if (pos == -1) { // error
-                        qWarning() << "kioslave bug: not enough slashes in UDS_URL" << path << "- looking for" << numberOfSlashes << "slashes";
+                        qCWarning(KIO_CORE) << "kioslave bug: not enough slashes in UDS_URL" << path << "- looking for" << numberOfSlashes << "slashes";
                         break;
                     }
                 }
@@ -1524,20 +1545,20 @@ KIO::Job *CopyJobPrivate::linkNextFile(const QUrl &uSource, const QUrl &uDest, J
                 m_processedFiles++;
                 //emit processedAmount( this, KJob::Files, m_processedFiles );
                 copyNextFile();
-                return 0;
+                return nullptr;
             } else {
                 qCDebug(KIO_COPYJOB_DEBUG) << "ERR_CANNOT_OPEN_FOR_WRITING";
                 q->setError(ERR_CANNOT_OPEN_FOR_WRITING);
                 q->setErrorText(uDest.toLocalFile());
                 q->emitResult();
-                return 0;
+                return nullptr;
             }
         } else {
             // Todo: not show "link" on remote dirs if the src urls are not from the same protocol+host+...
             q->setError(ERR_CANNOT_SYMLINK);
             q->setErrorText(uDest.toDisplayString());
             q->emitResult();
-            return 0;
+            return nullptr;
         }
     }
 }
@@ -1592,7 +1613,7 @@ void CopyJobPrivate::copyNextFile()
         const JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
 
         m_bCurrentOperationIsLink = false;
-        KIO::Job *newjob = 0;
+        KIO::Job *newjob = nullptr;
         if (m_mode == CopyJob::Link) {
             // User requested that a symlink be made
             newjob = linkNextFile(uSource, uDest, flags);
@@ -1843,7 +1864,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                 QTemporaryFile tmpFile(srcDir + "kio_XXXXXX");
                 const bool openOk = tmpFile.open();
                 if (!openOk) {
-                    qWarning() << "Couldn't open temp file in" << srcDir;
+                    qCWarning(KIO_CORE) << "Couldn't open temp file in" << srcDir;
                 } else {
                     const QString _tmp(tmpFile.fileName());
                     tmpFile.close();
@@ -1858,7 +1879,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                             qCDebug(KIO_COPYJOB_DEBUG) << "Didn't manage to rename" << _tmp << "to" << _dest << ", reverting";
                             // Revert back to original name!
                             if (!QFile::rename(_tmp, _src)) {
-                                qWarning() << "Couldn't rename" << _tmp << "back to" << _src << '!';
+                                qCWarning(KIO_CORE) << "Couldn't rename" << _tmp << "back to" << _src << '!';
                                 // Severe error, abort
                                 q->Job::slotResult(job); // will set the error and emit result(this)
                                 return;
