@@ -159,6 +159,15 @@ public:
      */
     void setUrl(const QUrl &url);
 
+    /**
+     * Sets the item's local path (UDS_LOCAL_PATH). Do not call unless you know what you are doing!
+     * This won't change the item's name or URL.
+     * (used for example when an item got renamed).
+     * @param path the item's local path
+     * @since 5.20
+     */
+    void setLocalPath(const QString &path);
+
     /**
      * Sets the item's name (i.e. the filename).
      * This is automatically done by setUrl, to set the name from the URL's fileName().
@@ -480,7 +489,7 @@ public:
      * The given boolean indicates if the returned url is local or not.
      * \since 4.6
      */
-    QUrl mostLocalUrl(bool *local = 0) const;
+    QUrl mostLocalUrl(bool *local = nullptr) const;
 
     /**
      * @deprecated since 5.0 add '&' in front of your boolean argument
