@@ -78,9 +78,9 @@ class KNewFileMenuSingleton
 {
 public:
     KNewFileMenuSingleton()
-        : dirWatch(0),
+        : dirWatch(nullptr),
           filesParsed(false),
-          templatesList(0),
+          templatesList(nullptr),
           templatesVersion(0)
     {
     }
@@ -312,11 +312,15 @@ public:
     void _k_slotFillTemplates();
 
     /**
-      * Callback in KNewFileMenu for the OtherDesktopFile Dialog. Handles dialog input and gives over
-      * to executeStrategy()
+      * Called when accepting the KPropertiesDialog (for "other desktop files")
       */
     void _k_slotOtherDesktopFile();
 
+    /**
+     * Called when closing the KPropertiesDialog is closed (whichever way, accepted and rejected)
+     */
+    void _k_slotOtherDesktopFileClosed();
+
     /**
       * Callback in KNewFileMenu for the RealFile Dialog. Handles dialog input and gives over
       * to executeStrategy()
@@ -387,7 +391,7 @@ bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
 
         KMessageBox::createKMessageBox(dialog, buttonBox, QMessageBox::Warning,
                                        i18n("<qt>The template file <b>%1</b> does not exist.</qt>", src),
-                                       QStringList(), QString(), 0, KMessageBox::NoExec,
+                                       QStringList(), QString(), nullptr, KMessageBox::NoExec,
                                        QString());
 
         dialog->show();
@@ -423,7 +427,7 @@ void KNewFileMenuPrivate::confirmCreatingHiddenDir(const QString &name)
                                    i18n("The name \"%1\" starts with a dot, so the directory will be hidden by default.", name),
                                    QStringList(),
                                    i18n("Do not ask again"),
-                                   0,
+                                   nullptr,
                                    KMessageBox::NoExec,
                                    QString());
 
@@ -448,19 +452,39 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         text = text.trimmed(); // In some languages, there is a space in front of "...", see bug 268895
         // KDE5 TODO: remove the "..." from link*.desktop files and use i18n("%1...") when making
         // the action.
+        QString name = text;
+        text.append(QStringLiteral(".desktop"));
 
-        QUrl defaultFile(*it);
-        defaultFile.setPath(defaultFile.path() + '/' + KIO::encodeFileName(text));
+        const QUrl directory = mostLocalUrl(*it);
+        const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + '/' + KIO::encodeFileName(text));
         if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
-            text = KIO::suggestName(*it, text);
+            text = KIO::suggestName(directory, text);
         }
 
-        const QUrl templateUrl(QUrl::fromLocalFile(entry.templatePath));
+        QUrl templateUrl;
+        bool usingTemplate = false;
+        if (entry.templatePath.startsWith(QLatin1String(":/"))) {
+            QTemporaryFile *tmpFile = QTemporaryFile::createNativeFile(entry.templatePath);
+            tmpFile->setAutoRemove(false);
+            QString tempFileName = tmpFile->fileName();
+            tmpFile->close();
 
-        QDialog *dlg = new KPropertiesDialog(templateUrl, *it, text, m_parentWidget);
+            KDesktopFile df(tempFileName);
+            KConfigGroup group = df.desktopGroup();
+            group.writeEntry("Name", name);
+            templateUrl = QUrl::fromLocalFile(tempFileName);
+            m_tempFileToDelete = tempFileName;
+            usingTemplate = true;
+        } else {
+            templateUrl = QUrl::fromLocalFile(entry.templatePath);
+        }
+        QDialog *dlg = new KPropertiesDialog(templateUrl, directory, text, m_parentWidget);
         dlg->setModal(q->isModal());
         dlg->setAttribute(Qt::WA_DeleteOnClose);
         QObject::connect(dlg, SIGNAL(applied()), q, SLOT(_k_slotOtherDesktopFile()));
+        if (usingTemplate) {
+            QObject::connect(dlg, SIGNAL(propertiesClosed()), q, SLOT(_k_slotOtherDesktopFileClosed()));
+        }
         dlg->show();
     }
     // We don't set m_src here -> there will be no copy, we are done.
@@ -475,10 +499,10 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
     text = text.trimmed(); // In some languages, there is a space in front of "...", see bug 268895
     m_copyData.m_src = entry.templatePath;
 
-    QUrl defaultFile(m_popupFiles.first());
-    defaultFile.setPath(defaultFile.path() + '/' + KIO::encodeFileName(text));
+    const QUrl directory = mostLocalUrl(m_popupFiles.first());
+    const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + '/' + KIO::encodeFileName(text));
     if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
-        text = KIO::suggestName(m_popupFiles.first(), text);
+        text = KIO::suggestName(directory, text);
     }
 
     QDialog *fileDialog = new QDialog(m_parentWidget);
@@ -569,16 +593,17 @@ void KNewFileMenuPrivate::executeStrategy()
         lstSrc.append(uSrc);
         KIO::Job *kjob;
         if (m_copyData.m_isSymlink) {
-            kjob = KIO::symlink(src, dest);
-            // This doesn't work, FileUndoManager registers new links in copyingLinkDone,
-            // which KIO::symlink obviously doesn't emit... Needs code in FileUndoManager.
-            //KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Link, lstSrc, dest, kjob);
+            KIO::CopyJob *linkJob = KIO::linkAs(uSrc, dest);
+            kjob = linkJob;
+            KIO::FileUndoManager::self()->recordCopyJob(linkJob);
         } else if (src.startsWith(QLatin1String(":/"))) {
             QFile srcFile(src);
             if (!srcFile.open(QIODevice::ReadOnly)) {
                 return;
             }
-            KIO::StoredTransferJob* putJob = KIO::storedPut(&srcFile, dest, -1);
+            // The QFile won't live long enough for the job, so let's buffer the contents
+            const QByteArray srcBuf(srcFile.readAll());
+            KIO::StoredTransferJob* putJob = KIO::storedPut(srcBuf, dest, -1);
             kjob = putJob;
             KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Put, QList<QUrl>(), dest, putJob);
         } else {
@@ -586,7 +611,7 @@ void KNewFileMenuPrivate::executeStrategy()
             KIO::CopyJob *job = KIO::copyAs(uSrc, dest);
             job->setDefaultPermissions(true);
             kjob = job;
-            KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Copy, lstSrc, dest, job);
+            KIO::FileUndoManager::self()->recordCopyJob(job);
         }
         KJobWidgets::setWindow(kjob, m_parentWidget);
         QObject::connect(kjob, SIGNAL(result(KJob*)), q, SLOT(slotResult(KJob*)));
@@ -610,13 +635,13 @@ void KNewFileMenuPrivate::fillMenu()
     QMenu *menu = q->menu();
     menu->clear();
     m_menuDev->menu()->clear();
-    m_newDirAction = 0;
+    m_newDirAction = nullptr;
 
     QSet<QString> seenTexts;
     // these shall be put at special positions
-    QAction *linkURL = 0;
-    QAction *linkApp = 0;
-    QAction *linkPath = 0;
+    QAction *linkURL = nullptr;
+    QAction *linkApp = nullptr;
+    QAction *linkPath = nullptr;
 
     KNewFileMenuSingleton *s = kNewMenuGlobals();
     int i = 1;
@@ -646,6 +671,19 @@ void KNewFileMenuPrivate::fillMenu()
                     act->setIcon(QIcon::fromTheme(entry.icon));
                     act->setText(i18nc("@item:inmenu Create New", "%1", entry.text));
                     act->setActionGroup(m_newMenuGroup);
+
+                    // If there is a shortcut available in the action collection, use it.
+                    QAction *act2 = m_actionCollection->action(QStringLiteral("create_dir"));
+                    if (act2) {
+                        act->setShortcuts(act2->shortcuts());
+                        // Both actions have now the same shortcut, so this will prevent the "Ambiguous shortcut detected" dialog.
+                        act->setShortcutContext(Qt::WidgetShortcut);
+                        // We also need to react to shortcut changes.
+                        QObject::connect(act2, &QAction::changed, act, [=]() {
+                            act->setShortcuts(act2->shortcuts());
+                        });
+                    }
+
                     menu->addAction(act);
 
                     QAction *sep = new QAction(q);
@@ -843,6 +881,11 @@ void KNewFileMenuPrivate::_k_slotCreateHiddenDirectory()
     _k_slotCreateDirectory(true);
 }
 
+struct EntryWithName {
+    QString key;
+    KNewFileMenuSingleton::Entry entry;
+};
+
 void KNewFileMenuPrivate::_k_slotFillTemplates()
 {
     KNewFileMenuSingleton *s = kNewMenuGlobals();
@@ -885,6 +928,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     }
 
     QMap<QString, KNewFileMenuSingleton::Entry> slist; // used for sorting
+    QMap<QString, EntryWithName> ulist; // entries with unique URLs
     Q_FOREACH (const QString &file, files) {
         //qDebug() << file;
         if (file[0] != '.') {
@@ -897,6 +941,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
             // This also sorts by user-visible name.
             // The rest of the re-ordering is done in fillMenu.
             const KDesktopFile config(file);
+            QString url = config.desktopGroup().readEntry("URL");
             QString key = config.desktopGroup().readEntry("Name");
             if (file.endsWith(QLatin1String("Directory.desktop"))) {
                 key.prepend('0');
@@ -905,15 +950,31 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
             } else {
                 key.prepend('2');
             }
-            slist.insert(key, e);
+            EntryWithName en = { key, e };
+            if (ulist.contains(url)) {
+                ulist.remove(url);
+            }
+            ulist.insert(url, en);
         }
     }
+    QMap<QString, EntryWithName>::iterator it = ulist.begin();
+    for (; it != ulist.end(); ++it) {
+        EntryWithName ewn = *it;
+        slist.insert(ewn.key, ewn.entry);
+    }
     (*s->templatesList) += slist.values();
 }
 
 void KNewFileMenuPrivate::_k_slotOtherDesktopFile()
 {
-    executeStrategy();
+    // The properties dialog took care of the copying, so we're done
+    KPropertiesDialog *dialog = qobject_cast<KPropertiesDialog *>(q->sender());
+    emit q->fileCreated(dialog->url());
+}
+
+void KNewFileMenuPrivate::_k_slotOtherDesktopFileClosed()
+{
+    QFile::remove(m_tempFileToDelete);
 }
 
 void KNewFileMenuPrivate::_k_slotRealFileOrDir()
@@ -928,36 +989,13 @@ void KNewFileMenuPrivate::_k_slotSymLink()
     KNameAndUrlInputDialog *dlg = static_cast<KNameAndUrlInputDialog *>(m_fileDialog);
 
     m_copyData.m_chosenFileName = dlg->name(); // no path
-    QUrl linkUrl = dlg->url(); // the url to put in the file
+    const QString linkTarget = dlg->urlText();
 
-    if (m_copyData.m_chosenFileName.isEmpty() || linkUrl.isEmpty()) {
+    if (m_copyData.m_chosenFileName.isEmpty() || linkTarget.isEmpty()) {
         return;
     }
 
-    if (linkUrl.isRelative()) {
-        m_copyData.m_src = linkUrl.url();
-    } else if (linkUrl.isLocalFile()) {
-        m_copyData.m_src = linkUrl.toLocalFile();
-    } else {
-        QDialog *dialog = new QDialog(m_parentWidget);
-        dialog->setWindowTitle(i18n("Sorry"));
-        dialog->setObjectName(QStringLiteral("sorry"));
-        dialog->setModal(m_modal);
-        dialog->setAttribute(Qt::WA_DeleteOnClose);
-
-        QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
-        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
-
-        m_fileDialog = dialog;
-
-        KMessageBox::createKMessageBox(dialog, buttonBox, QMessageBox::Warning,
-                                       i18n("Basic links can only point to local files or directories.\nPlease use \"Link to Location\" for remote URLs."),
-                                       QStringList(), QString(), 0, KMessageBox::NoExec,
-                                       QString());
-
-        dialog->show();
-        return;
-    }
+    m_copyData.m_src = linkTarget;
     executeStrategy();
 }
 
@@ -1037,7 +1075,7 @@ KNewFileMenu::KNewFileMenu(KActionCollection *collection, const QString &name, Q
     connect(d->m_newMenuGroup, SIGNAL(triggered(QAction*)), this, SLOT(_k_slotActionTriggered(QAction*)));
     d->m_actionCollection = collection;
     d->m_parentWidget = qobject_cast<QWidget *>(parent);
-    d->m_newDirAction = 0;
+    d->m_newDirAction = nullptr;
 
     if (d->m_actionCollection) {
         d->m_actionCollection->addAction(name, this);
@@ -1199,17 +1237,19 @@ void KNewFileMenu::slotResult(KJob *job)
             const QUrl localUrl = d->mostLocalUrl(destUrl);
             if (localUrl.isLocalFile()) {
                 // Normal (local) file. Need to "touch" it, kio_file copied the mtime.
-                (void) ::utime(QFile::encodeName(localUrl.toLocalFile()).constData(), 0);
+                (void) ::utime(QFile::encodeName(localUrl.toLocalFile()).constData(), nullptr);
             }
             emit fileCreated(destUrl);
         } else if (KIO::SimpleJob *simpleJob = ::qobject_cast<KIO::SimpleJob *>(job)) {
-            // we just created a symlink
+            // Called in the storedPut() case
             emit fileCreated(simpleJob->url());
-        } else {
-            // Can be mkdir or symlink
-            QUrl mkpathUrl = job->property("mkpathUrl").value<QUrl>();
+        }  else {
+            // Can be mkdir
+            QUrl mkpathUrl = job->property("mkpathUrl").toUrl();
             if (mkpathUrl.isValid()) {
                 emit directoryCreated(mkpathUrl);
+            } else {
+                qWarning() << "Neither copy, put nor mkdir, internal error";
             }
         }
     }
