@@ -55,7 +55,7 @@ int initLocale()
 {
 #ifdef UTF8TEST
     // Assume utf8 system
-    setenv("LC_ALL", "en_US.utf-8", 1);
+    setenv("LC_ALL", "C.utf-8", 1);
     setenv("KDE_UTF8_FILENAMES", "true", 1);
 #else
     // Ensure a known QFile::encodeName behavior for trashUtf8FileFromHome
@@ -139,14 +139,22 @@ void TestTrash::initTestCase()
 
     QVERIFY(m_tempDir.isValid());
 
+#ifndef Q_OS_OSX
     m_trashDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QStringLiteral("/Trash");
     qDebug() << "setup: using trash directory " << m_trashDir;
+#endif
 
     // Look for another writable partition than $HOME (not mandatory)
     TrashImpl impl;
     impl.init();
 
     TrashImpl::TrashDirMap trashDirs = impl.trashDirectories();
+#ifdef Q_OS_OSX
+    QVERIFY(trashDirs.contains(0));
+    m_trashDir = trashDirs.value(0);
+    qDebug() << "setup: using trash directory " << m_trashDir;
+#endif
+
     TrashImpl::TrashDirMap topDirs = impl.topDirectories();
     bool foundTrashDir = false;
     m_otherPartitionId = 0;
@@ -452,8 +460,12 @@ void TestTrash::trashFileOwnedByRoot()
     QUrl u(QStringLiteral("file:///etc/passwd"));
     const QString fileId = QStringLiteral("passwd");
 
+    if (geteuid() == 0 || QFileInfo(u.toLocalFile()).isWritable()) {
+        QSKIP("Test must not be run by root.");
+    }
+
     KIO::CopyJob *job = KIO::move(u, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
-    job->setUiDelegate(0); // no skip dialog, thanks
+    job->setUiDelegate(nullptr); // no skip dialog, thanks
     bool ok = job->exec();
     QVERIFY(!ok);
     QMap<QString, QString> metaData = job->metaData();
@@ -957,8 +969,12 @@ void TestTrash::trashDirectoryOwnedByRoot()
     const QString fileId = u.path();
     qDebug() << "fileId=" << fileId;
 
+    if (geteuid() == 0 || QFileInfo(u.toLocalFile()).isWritable()) {
+        QSKIP("Test must not be run by root.");
+    }
+
     KIO::CopyJob *job = KIO::move(u, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
-    job->setUiDelegate(0); // no skip dialog, thanks
+    job->setUiDelegate(nullptr); // no skip dialog, thanks
     bool ok = job->exec();
     QVERIFY(!ok);
     QMap<QString, QString> metaData = job->metaData();
