@@ -48,7 +48,7 @@ public:
      *               directory where a file has been selected.
      * @param parent The parent object of this widget.
      */
-    explicit KUrlRequesterDialog(const QUrl &url, QWidget *parent = 0);
+    explicit KUrlRequesterDialog(const QUrl &url, QWidget *parent = nullptr);
 
     /**
      * Constructs a KUrlRequesterDialog.
@@ -79,11 +79,11 @@ public:
      * @param caption The caption to use for the dialog.
      */
     static QUrl getUrl(const QUrl &url = QUrl(),
-                       QWidget *parent = 0, const QString &caption = QString());
+                       QWidget *parent = nullptr, const QString &caption = QString());
 
     /**
      * Returns a pointer to the file dialog used by the KUrlRequester.
-     * @deprecated since 5.0
+     * @deprecated since 5.0, use urlRequester() methods instead.
      */
 #ifndef KIOWIDGETS_NO_DEPRECATED
     KIOWIDGETS_DEPRECATED QFileDialog *fileDialog();
@@ -100,7 +100,6 @@ private:
 
     Q_DISABLE_COPY(KUrlRequesterDialog)
 
-    Q_PRIVATE_SLOT(d, void _k_slotClear())
     Q_PRIVATE_SLOT(d, void _k_slotTextChanged(const QString &))
 };
 
